@@ -33,8 +33,9 @@ class ClassMapGenerator
     /**
      * Generate a class map file
      *
-     * @param \Traversable $dirs Directories or a single path to search in
-     * @param string       $file The name of the class map file
+     * @param \Traversable<string>|array<string> $dirs Directories or a single path to search in
+     * @param string                     $file The name of the class map file
+     * @return void
      */
     public static function dump($dirs, $file)
     {
@@ -50,14 +51,14 @@ class ClassMapGenerator
     /**
      * Iterate over all files in the given directory searching for classes
      *
-     * @param \Iterator|string $path         The path to search in or an iterator
-     * @param string           $excluded     Regex that matches file paths to be excluded from the classmap
-     * @param IOInterface      $io           IO object
-     * @param string           $namespace    Optional namespace prefix to filter by
-     * @param string           $autoloadType psr-0|psr-4 Optional autoload standard to use mapping rules
-     *
+     * @param \Traversable<\SplFileInfo>|string|array<string> $path The path to search in or an iterator
+     * @param string              $excluded     Regex that matches file paths to be excluded from the classmap
+     * @param ?IOInterface        $io           IO object
+     * @param ?string             $namespace    Optional namespace prefix to filter by
+     * @param ?string             $autoloadType psr-0|psr-4 Optional autoload standard to use mapping rules
+     * @param array<string, true> $scannedFiles
+     * @return array<class-string, string> A class map array
      * @throws \RuntimeException When the path is neither an existing file nor directory
-     * @return array             A class map array
      */
     public static function createMap($path, $excluded = null, IOInterface $io = null, $namespace = null, $autoloadType = null, &$scannedFiles = array())
     {
@@ -147,13 +148,13 @@ class ClassMapGenerator
     /**
      * Remove classes which could not have been loaded by namespace autoloaders
      *
-     * @param  array       $classes       found classes in given file
-     * @param  string      $filePath      current file
-     * @param  string      $baseNamespace prefix of given autoload mapping
-     * @param  string      $namespaceType psr-0|psr-4
-     * @param  string      $basePath      root directory of given autoload mapping
-     * @param  IOInterface $io            IO object
-     * @return array       valid classes
+     * @param  array<int, class-string> $classes       found classes in given file
+     * @param  string                   $filePath      current file
+     * @param  string                   $baseNamespace prefix of given autoload mapping
+     * @param  string                   $namespaceType psr-0|psr-4
+     * @param  string                   $basePath      root directory of given autoload mapping
+     * @param  ?IOInterface             $io            IO object
+     * @return array<int, class-string> valid classes
      */
     private static function filterByNamespace($classes, $filePath, $baseNamespace, $namespaceType, $basePath, $io)
     {
@@ -210,14 +211,11 @@ class ClassMapGenerator
      *
      * @param  string            $path The file to check
      * @throws \RuntimeException
-     * @return array             The found classes
+     * @return array<int, class-string> The found classes
      */
     private static function findClasses($path)
     {
-        $extraTypes = PHP_VERSION_ID < 50400 ? '' : '|trait';
-        if (defined('HHVM_VERSION') && version_compare(HHVM_VERSION, '3.3', '>=')) {
-            $extraTypes .= '|enum';
-        }
+        $extraTypes = self::getExtraTypes();
 
         // Use @ here instead of Silencer to actively suppress 'unhelpful' output
         // @link https://github.com/composer/composer/pull/4886
@@ -225,7 +223,7 @@ class ClassMapGenerator
         if (!$contents) {
             if (!file_exists($path)) {
                 $message = 'File at "%s" does not exist, check your classmap definitions';
-            } elseif (!is_readable($path)) {
+            } elseif (!Filesystem::isReadable($path)) {
                 $message = 'File at "%s" is not readable, check its permissions';
             } elseif ('' === trim(file_get_contents($path))) {
                 // The input file was really empty and thus contains no classes
@@ -241,32 +239,14 @@ class ClassMapGenerator
         }
 
         // return early if there is no chance of matching anything in this file
-        if (!preg_match('{\b(?:class|interface'.$extraTypes.')\s}i', $contents)) {
+        preg_match_all('{\b(?:class|interface'.$extraTypes.')\s}i', $contents, $matches);
+        if (!$matches) {
             return array();
         }
 
-        // strip heredocs/nowdocs
-        $contents = preg_replace('{<<<[ \t]*([\'"]?)(\w+)\\1(?:\r\n|\n|\r)(?:.*?)(?:\r\n|\n|\r)(?:\s*)\\2(?=\s+|[;,.)])}s', 'null', $contents);
-        // strip strings
-        $contents = preg_replace('{"[^"\\\\]*+(\\\\.[^"\\\\]*+)*+"|\'[^\'\\\\]*+(\\\\.[^\'\\\\]*+)*+\'}s', 'null', $contents);
-        // strip leading non-php code if needed
-        if (strpos($contents, '<?') !== 0) {
-            $contents = preg_replace('{^.+?<\?}s', '<?', $contents, 1, $replacements);
-            if ($replacements === 0) {
-                return array();
-            }
-        }
-        // strip non-php blocks in the file
-        $contents = preg_replace('{\?>(?:[^<]++|<(?!\?))*+<\?}s', '?><?', $contents);
-        // strip trailing non-php code if needed
-        $pos = strrpos($contents, '?>');
-        if (false !== $pos && false === strpos(substr($contents, $pos), '<?')) {
-            $contents = substr($contents, 0, $pos);
-        }
-        // strip comments if short open tags are in the file
-        if (preg_match('{(<\?)(?!(php|hh))}i', $contents)) {
-            $contents = preg_replace('{//.* | /\*(?:[^*]++|\*(?!/))*\*/}x', '', $contents);
-        }
+        $p = new PhpFileCleaner($contents, count($matches[0]));
+        $contents = $p->clean();
+        unset($p);
 
         preg_match_all('{
             (?:
@@ -303,4 +283,21 @@ class ClassMapGenerator
 
         return $classes;
     }
+
+    /**
+     * @return string
+     */
+    private static function getExtraTypes()
+    {
+        static $extraTypes = null;
+        if (null === $extraTypes) {
+            $extraTypes = PHP_VERSION_ID < 50400 ? '' : '|trait';
+            if (PHP_VERSION_ID >= 80100 || (defined('HHVM_VERSION') && version_compare(HHVM_VERSION, '3.3', '>='))) {
+                $extraTypes .= '|enum';
+            }
+            PhpFileCleaner::setTypeConfig(array_merge(array('class', 'interface'), array_filter(explode('|', $extraTypes))));
+        }
+
+        return $extraTypes;
+    }
 }
