@@ -13,6 +13,7 @@
 namespace Composer\Console;
 
 use Composer\IO\NullIO;
+use Composer\Util\Filesystem;
 use Composer\Util\Platform;
 use Composer\Util\Silencer;
 use Symfony\Component\Console\Application as BaseApplication;
@@ -33,6 +34,8 @@ use Composer\Util\ErrorHandler;
 use Composer\Util\HttpDownloader;
 use Composer\EventDispatcher\ScriptExecutionException;
 use Composer\Exception\NoSslException;
+use Composer\XdebugHandler\XdebugHandler;
+use Symfony\Component\Process\Exception\ProcessTimedOutException;
 
 /**
  * The console application that handles the commands
@@ -44,7 +47,7 @@ use Composer\Exception\NoSslException;
 class Application extends BaseApplication
 {
     /**
-     * @var Composer
+     * @var ?Composer
      */
     protected $composer;
 
@@ -53,6 +56,7 @@ class Application extends BaseApplication
      */
     protected $io;
 
+    /** @var string */
     private static $logo = '   ______
   / ____/___  ____ ___  ____  ____  ________  _____
  / /   / __ \/ __ `__ \/ __ \/ __ \/ ___/ _ \/ ___/
@@ -61,7 +65,9 @@ class Application extends BaseApplication
                     /_/
 ';
 
+    /** @var bool */
     private $hasPluginCommands = false;
+    /** @var bool */
     private $disablePluginsByDefault = false;
 
     /**
@@ -74,8 +80,8 @@ class Application extends BaseApplication
         static $shutdownRegistered = false;
 
         if (function_exists('ini_set') && extension_loaded('xdebug')) {
-            ini_set('xdebug.show_exception_trace', false);
-            ini_set('xdebug.scream', false);
+            ini_set('xdebug.show_exception_trace', '0');
+            ini_set('xdebug.scream', '0');
         }
 
         if (function_exists('date_default_timezone_set') && function_exists('date_default_timezone_get')) {
@@ -111,7 +117,7 @@ class Application extends BaseApplication
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function run(InputInterface $input = null, OutputInterface $output = null)
     {
@@ -123,7 +129,7 @@ class Application extends BaseApplication
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function doRun(InputInterface $input, OutputInterface $output)
     {
@@ -136,12 +142,13 @@ class Application extends BaseApplication
         $io = $this->io = new ConsoleIO($input, $output, new HelperSet(array(
             new QuestionHelper(),
         )));
+
+        // Register error handler again to pass it the IO instance
         ErrorHandler::register($io);
 
         if ($input->hasParameterOption('--no-cache')) {
             $io->writeError('Disabling cache usage', true, IOInterface::DEBUG);
-            $_SERVER['COMPOSER_CACHE_DIR'] = Platform::isWindows() ? 'nul' : '/dev/null';
-            putenv('COMPOSER_CACHE_DIR='.$_SERVER['COMPOSER_CACHE_DIR']);
+            Platform::putEnv('COMPOSER_CACHE_DIR', Platform::isWindows() ? 'nul' : '/dev/null');
         }
 
         // switch working dir
@@ -182,7 +189,17 @@ class Application extends BaseApplication
             }
         }
 
-        if (!$this->disablePluginsByDefault && !$this->hasPluginCommands && 'global' !== $commandName) {
+        // avoid loading plugins/initializing the Composer instance earlier than necessary if no plugin command is needed
+        // if showing the version, we never need plugin commands
+        $mayNeedPluginCommand = false === $input->hasParameterOption(array('--version', '-V'))
+            && (
+                // not a composer command, so try loading plugin ones
+                false === $commandName
+                // list command requires plugin commands to show them
+                || in_array($commandName, array('', 'list', 'help'), true)
+            );
+
+        if ($mayNeedPluginCommand && !$this->disablePluginsByDefault && !$this->hasPluginCommands) {
             try {
                 foreach ($this->getPluginCommands() as $command) {
                     if ($this->has($command->getName())) {
@@ -236,7 +253,7 @@ class Application extends BaseApplication
                 $io->writeError('<warning>Composer only officially supports PHP 5.3.2 and above, you will most likely encounter problems with your PHP '.PHP_VERSION.', upgrading is strongly recommended.</warning>');
             }
 
-            if (extension_loaded('xdebug') && !getenv('COMPOSER_DISABLE_XDEBUG_WARN')) {
+            if (XdebugHandler::isXdebugActive() && !getenv('COMPOSER_DISABLE_XDEBUG_WARN')) {
                 $io->writeError('<warning>Composer is operating slower than normal because you have Xdebug enabled. See https://getcomposer.org/xdebug</warning>');
             }
 
@@ -280,7 +297,7 @@ class Application extends BaseApplication
 
             // add non-standard scripts as own commands
             $file = Factory::getComposerFile();
-            if (is_file($file) && is_readable($file) && is_array($composer = json_decode(file_get_contents($file), true))) {
+            if (is_file($file) && Filesystem::isReadable($file) && is_array($composer = json_decode(file_get_contents($file), true))) {
                 if (isset($composer['scripts']) && is_array($composer['scripts'])) {
                     foreach ($composer['scripts'] as $script => $dummy) {
                         if (!defined('Composer\Script\ScriptEvents::'.str_replace('-', '_', strtoupper($script)))) {
@@ -351,9 +368,9 @@ class Application extends BaseApplication
     }
 
     /**
-     * {@inheritDoc}
+     * @return void
      */
-    private function hintCommonErrors($exception)
+    private function hintCommonErrors(\Exception $exception)
     {
         $io = $this->getIO();
 
@@ -385,6 +402,11 @@ class Application extends BaseApplication
             $io->writeError('<error>Check https://getcomposer.org/doc/articles/troubleshooting.md#proc-open-fork-failed-errors for details</error>', true, IOInterface::QUIET);
         }
 
+        if ($exception instanceof ProcessTimedOutException) {
+            $io->writeError('<error>The following exception is caused by a process timeout</error>', true, IOInterface::QUIET);
+            $io->writeError('<error>Check https://getcomposer.org/doc/06-config.md#process-timeout for details</error>', true, IOInterface::QUIET);
+        }
+
         if ($hints = HttpDownloader::getExceptionHints($exception)) {
             foreach ($hints as $hint) {
                 $io->writeError($hint, true, IOInterface::QUIET);
@@ -396,7 +418,8 @@ class Application extends BaseApplication
      * @param  bool                    $required
      * @param  bool|null               $disablePlugins
      * @throws JsonValidationException
-     * @return \Composer\Composer
+     * @throws \InvalidArgumentException
+     * @return ?\Composer\Composer If $required is true then the return value is guaranteed
      */
     public function getComposer($required = true, $disablePlugins = null)
     {
@@ -410,12 +433,16 @@ class Application extends BaseApplication
             } catch (\InvalidArgumentException $e) {
                 if ($required) {
                     $this->io->writeError($e->getMessage());
-                    exit(1);
+                    // TODO composer 2.3 simplify to $this->areExceptionsCaught()
+                    if (!method_exists($this, 'areExceptionsCaught') || $this->areExceptionsCaught()) {
+                        exit(1);
+                    }
+                    throw $e;
                 }
             } catch (JsonValidationException $e) {
-                $errors = ' - ' . implode(PHP_EOL . ' - ', $e->getErrors());
-                $message = $e->getMessage() . ':' . PHP_EOL . $errors;
-                throw new JsonValidationException($message);
+                if ($required) {
+                    throw $e;
+                }
             }
         }
 
@@ -424,11 +451,13 @@ class Application extends BaseApplication
 
     /**
      * Removes the cached composer instance
+     *
+     * @return void
      */
     public function resetComposer()
     {
         $this->composer = null;
-        if ($this->getIO() && method_exists($this->getIO(), 'resetAuthentications')) {
+        if (method_exists($this->getIO(), 'resetAuthentications')) {
             $this->getIO()->resetAuthentications();
         }
     }
@@ -441,6 +470,9 @@ class Application extends BaseApplication
         return $this->io;
     }
 
+    /**
+     * @return string
+     */
     public function getHelp()
     {
         return self::$logo . parent::getHelp();
@@ -479,6 +511,7 @@ class Application extends BaseApplication
             new Command\OutdatedCommand(),
             new Command\CheckPlatformReqsCommand(),
             new Command\FundCommand(),
+            new Command\ReinstallCommand(),
         ));
 
         if (strpos(__FILE__, 'phar:') === 0) {
@@ -489,7 +522,7 @@ class Application extends BaseApplication
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     public function getLongVersion()
     {
@@ -507,7 +540,7 @@ class Application extends BaseApplication
     }
 
     /**
-     * {@inheritDoc}
+     * @inheritDoc
      */
     protected function getDefaultInputDefinition()
     {
@@ -520,6 +553,9 @@ class Application extends BaseApplication
         return $definition;
     }
 
+    /**
+     * @return Command\BaseCommand[]
+     */
     private function getPluginCommands()
     {
         $commands = array();
