@@ -16,11 +16,14 @@ use Composer\EventDispatcher\EventDispatcher;
 use Composer\IO\IOInterface;
 use Composer\Package\AliasPackage;
 use Composer\Package\BasePackage;
+use Composer\Package\CompleteAliasPackage;
+use Composer\Package\CompletePackage;
 use Composer\Package\PackageInterface;
 use Composer\Package\Version\StabilityFilter;
 use Composer\Plugin\PluginEvents;
 use Composer\Plugin\PrePoolCreateEvent;
 use Composer\Repository\PlatformRepository;
+use Composer\Repository\RepositoryInterface;
 use Composer\Repository\RootPackageRepository;
 use Composer\Semver\CompilingMatcher;
 use Composer\Semver\Constraint\Constraint;
@@ -36,22 +39,26 @@ class PoolBuilder
 {
     /**
      * @var int[]
+     * @phpstan-var array<string, BasePackage::STABILITY_*>
      */
     private $acceptableStabilities;
     /**
      * @var int[]
+     * @phpstan-var array<string, BasePackage::STABILITY_*>
      */
     private $stabilityFlags;
     /**
-     * @psalm-var array<string, array<string, array{alias: string, alias_normalized: string}>>
+     * @var array[]
+     * @phpstan-var array<string, array<string, array{alias: string, alias_normalized: string}>>
      */
     private $rootAliases;
     /**
-     * @psalm-var array<string, string>
+     * @var string[]
+     * @phpstan-var array<string, string>
      */
     private $rootReferences;
     /**
-     * @var EventDispatcher
+     * @var ?EventDispatcher
      */
     private $eventDispatcher;
     /**
@@ -59,30 +66,36 @@ class PoolBuilder
      */
     private $io;
     /**
-     * @psalm-var array<string, AliasPackage>
+     * @var array[]
+     * @phpstan-var array<string, AliasPackage[]>
      */
     private $aliasMap = array();
     /**
-     * @psalm-var array<string, ConstraintInterface>
+     * @var ConstraintInterface[]
+     * @phpstan-var array<string, ConstraintInterface>
      */
     private $packagesToLoad = array();
     /**
-     * @psalm-var array<string, ConstraintInterface>
+     * @var ConstraintInterface[]
+     * @phpstan-var array<string, ConstraintInterface>
      */
     private $loadedPackages = array();
     /**
-     * @psalm-var array<int, array<string, array<string, PackageInterface>>>
+     * @var array[]
+     * @phpstan-var array<int, array<string, array<string, PackageInterface>>>
      */
     private $loadedPerRepo = array();
     /**
-     * @psalm-var Package[]
+     * @var BasePackage[]
      */
     private $packages = array();
     /**
-     * @psalm-var list<Package>
+     * @var BasePackage[]
      */
     private $unacceptableFixedOrLockedPackages = array();
+    /** @var string[] */
     private $updateAllowList = array();
+    /** @var array<string, string> */
     private $skippedLoad = array();
 
     /**
@@ -92,24 +105,28 @@ class PoolBuilder
      *
      * Packages get cleared from this list if they get unlocked as in that case
      * we need to actually load them
+     *
+     * @var array<string, true>
      */
     private $maxExtendedReqs = array();
     /**
-     * @psalm-var array<string, bool>
+     * @var array
+     * @phpstan-var array<string, bool>
      */
     private $updateAllowWarned = array();
 
+    /** @var int */
     private $indexCounter = 0;
 
     /**
      * @param int[] $acceptableStabilities array of stability => BasePackage::STABILITY_* value
-     * @psalm-param array<string, BasePackage::STABILITY_*> $acceptableStabilities
+     * @phpstan-param array<string, BasePackage::STABILITY_*> $acceptableStabilities
      * @param int[] $stabilityFlags an array of package name => BasePackage::STABILITY_* value
-     * @psalm-param array<string, BasePackage::STABILITY_*> $stabilityFlags
+     * @phpstan-param array<string, BasePackage::STABILITY_*> $stabilityFlags
      * @param array[] $rootAliases
-     * @psalm-param array<string, array<string, array{alias: string, alias_normalized: string}>> $rootAliases
+     * @phpstan-param array<string, array<string, array{alias: string, alias_normalized: string}>> $rootAliases
      * @param string[] $rootReferences an array of package name => source reference
-     * @psalm-param array<string, string> $rootReferences
+     * @phpstan-param array<string, string> $rootReferences
      */
     public function __construct(array $acceptableStabilities, array $stabilityFlags, array $rootAliases, array $rootReferences, IOInterface $io, EventDispatcher $eventDispatcher = null)
     {
@@ -121,6 +138,10 @@ class PoolBuilder
         $this->io = $io;
     }
 
+    /**
+     * @param RepositoryInterface[] $repositories
+     * @return Pool
+     */
     public function buildPool(array $repositories, Request $request)
     {
         if ($request->getUpdateAllowList()) {
@@ -243,6 +264,10 @@ class PoolBuilder
         return $pool;
     }
 
+    /**
+     * @param string $name
+     * @return void
+     */
     private function markPackageNameForLoading(Request $request, $name, ConstraintInterface $constraint)
     {
         // Skip platform requires at this stage
@@ -298,6 +323,10 @@ class PoolBuilder
         unset($this->loadedPackages[$name]);
     }
 
+    /**
+     * @param RepositoryInterface[] $repositories
+     * @return void
+     */
     private function loadPackagesMarkedForLoading(Request $request, $repositories)
     {
         foreach ($this->packagesToLoad as $name => $constraint) {
@@ -330,7 +359,11 @@ class PoolBuilder
         }
     }
 
-    private function loadPackage(Request $request, PackageInterface $package, $propagateUpdate = true)
+    /**
+     * @param bool $propagateUpdate
+     * @return void
+     */
+    private function loadPackage(Request $request, BasePackage $package, $propagateUpdate = true)
     {
         $index = $this->indexCounter++;
         $this->packages[$index] = $package;
@@ -360,7 +393,11 @@ class PoolBuilder
             } else {
                 $basePackage = $package;
             }
-            $aliasPackage = new AliasPackage($basePackage, $alias['alias_normalized'], $alias['alias']);
+            if ($basePackage instanceof CompletePackage) {
+                $aliasPackage = new CompleteAliasPackage($basePackage, $alias['alias_normalized'], $alias['alias']);
+            } else {
+                $aliasPackage = new AliasPackage($basePackage, $alias['alias_normalized'], $alias['alias']);
+            }
             $aliasPackage->setRootPackageAlias(true);
 
             $newIndex = $this->indexCounter++;
@@ -412,6 +449,7 @@ class PoolBuilder
     /**
      * Checks if a particular name is required directly in the request
      *
+     * @param string $name packageName
      * @return bool
      */
     private function isRootRequire(Request $request, $name)
@@ -423,10 +461,21 @@ class PoolBuilder
 
     /**
      * Checks whether the update allow list allows this package in the lock file to be updated
+     *
      * @return bool
      */
-    private function isUpdateAllowed(PackageInterface $package)
+    private function isUpdateAllowed(BasePackage $package)
     {
+        // Path repo packages are never loaded from lock, to force them to always remain in sync
+        // unless symlinking is disabled in which case we probably should rather treat them like
+        // regular packages
+        if ($package->getDistType() === 'path') {
+            $transportOptions = $package->getTransportOptions();
+            if (!isset($transportOptions['symlink']) || $transportOptions['symlink'] !== false) {
+                return true;
+            }
+        }
+
         foreach ($this->updateAllowList as $pattern => $void) {
             $patternRegexp = BasePackage::packageNameToRegexp($pattern);
             if (preg_match($patternRegexp, $package->getName())) {
@@ -437,6 +486,9 @@ class PoolBuilder
         return false;
     }
 
+    /**
+     * @return void
+     */
     private function warnAboutNonMatchingUpdateAllowList(Request $request)
     {
         foreach ($this->updateAllowList as $pattern => $void) {
@@ -464,10 +516,12 @@ class PoolBuilder
     /**
      * Reverts the decision to use a locked package if a partial update with transitive dependencies
      * found that this package actually needs to be updated
+     *
+     * @param string $name
+     * @return void
      */
     private function unlockPackage(Request $request, $name)
     {
-
         if (
             // if we unfixed a replaced package name, we also need to unfix the replacer itself
             $this->skippedLoad[$name] !== $name
@@ -503,7 +557,11 @@ class PoolBuilder
         }
     }
 
-    private function removeLoadedPackage(Request $request, PackageInterface $package, $index)
+    /**
+     * @param int $index
+     * @return void
+     */
+    private function removeLoadedPackage(Request $request, BasePackage $package, $index)
     {
         unset($this->packages[$index]);
         if (isset($this->aliasMap[spl_object_hash($package)])) {
