@@ -16,7 +16,6 @@ use Composer\IO\IOInterface;
 use Composer\IO\ConsoleIO;
 use Composer\Package\PackageInterface;
 use Composer\Package\AliasPackage;
-use Composer\Repository\RepositoryInterface;
 use Composer\Repository\InstalledRepositoryInterface;
 use Composer\DependencyResolver\Operation\OperationInterface;
 use Composer\DependencyResolver\Operation\InstallOperation;
@@ -24,8 +23,10 @@ use Composer\DependencyResolver\Operation\UpdateOperation;
 use Composer\DependencyResolver\Operation\UninstallOperation;
 use Composer\DependencyResolver\Operation\MarkAliasInstalledOperation;
 use Composer\DependencyResolver\Operation\MarkAliasUninstalledOperation;
+use Composer\Downloader\FileDownloader;
 use Composer\EventDispatcher\EventDispatcher;
 use Composer\Util\Loop;
+use Composer\Util\Platform;
 use React\Promise\PromiseInterface;
 
 /**
@@ -47,7 +48,7 @@ class InstallationManager
     private $loop;
     /** @var IOInterface */
     private $io;
-    /** @var EventDispatcher */
+    /** @var ?EventDispatcher */
     private $eventDispatcher;
     /** @var bool */
     private $outputProgress;
@@ -59,15 +60,21 @@ class InstallationManager
         $this->eventDispatcher = $eventDispatcher;
     }
 
+    /**
+     * @return void
+     */
     public function reset()
     {
         $this->notifiablePackages = array();
+        FileDownloader::$downloadMetadata = array();
     }
 
     /**
      * Adds installer
      *
      * @param InstallerInterface $installer installer instance
+     *
+     * @return void
      */
     public function addInstaller(InstallerInterface $installer)
     {
@@ -79,6 +86,8 @@ class InstallationManager
      * Removes installer
      *
      * @param InstallerInterface $installer installer instance
+     *
+     * @return void
      */
     public function removeInstaller(InstallerInterface $installer)
     {
@@ -94,6 +103,8 @@ class InstallationManager
      * We prevent any plugins from being instantiated by simply
      * deactivating the installer for them. This ensure that no third-party
      * code is ever executed.
+     *
+     * @return void
      */
     public function disablePlugins()
     {
@@ -153,6 +164,8 @@ class InstallationManager
      * If the installer associated to this package doesn't handle that function, it'll do nothing.
      *
      * @param PackageInterface $package Package instance
+     *
+     * @return void
      */
     public function ensureBinariesPresence(PackageInterface $package)
     {
@@ -172,13 +185,16 @@ class InstallationManager
     /**
      * Executes solver operation.
      *
-     * @param RepositoryInterface  $repo       repository in which to add/remove/update packages
-     * @param OperationInterface[] $operations operations to execute
-     * @param bool                 $devMode    whether the install is being run in dev mode
-     * @param bool                 $runScripts whether to dispatch script events
+     * @param InstalledRepositoryInterface $repo       repository in which to add/remove/update packages
+     * @param OperationInterface[]         $operations operations to execute
+     * @param bool                         $devMode    whether the install is being run in dev mode
+     * @param bool                         $runScripts whether to dispatch script events
+     *
+     * @return void
      */
-    public function execute(RepositoryInterface $repo, array $operations, $devMode = true, $runScripts = true)
+    public function execute(InstalledRepositoryInterface $repo, array $operations, $devMode = true, $runScripts = true)
     {
+        /** @var PromiseInterface[] */
         $cleanupPromises = array();
 
         $loop = $this->loop;
@@ -207,7 +223,7 @@ class InstallationManager
         };
 
         $handleInterruptsUnix = function_exists('pcntl_async_signals') && function_exists('pcntl_signal');
-        $handleInterruptsWindows = function_exists('sapi_windows_set_ctrl_handler');
+        $handleInterruptsWindows = function_exists('sapi_windows_set_ctrl_handler') && PHP_SAPI === 'cli';
         $prevHandler = null;
         $windowsHandler = null;
         if ($handleInterruptsUnix) {
@@ -243,8 +259,8 @@ class InstallationManager
             $batches = array();
             $batch = array();
             foreach ($operations as $index => $operation) {
-                if (in_array($operation->getOperationType(), array('update', 'install'), true)) {
-                    $package = $operation->getOperationType() === 'update' ? $operation->getTargetPackage() : $operation->getPackage();
+                if ($operation instanceof UpdateOperation || $operation instanceof InstallOperation) {
+                    $package = $operation instanceof UpdateOperation ? $operation->getTargetPackage() : $operation->getPackage();
                     if ($package->getType() === 'composer-plugin' && ($extra = $package->getExtra()) && isset($extra['plugin-modifies-downloads']) && $extra['plugin-modifies-downloads'] === true) {
                         if ($batch) {
                             $batches[] = $batch;
@@ -272,7 +288,7 @@ class InstallationManager
                 pcntl_signal(SIGINT, $prevHandler);
             }
             if ($handleInterruptsWindows) {
-                sapi_windows_set_ctrl_handler($prevHandler, false);
+                sapi_windows_set_ctrl_handler($windowsHandler, false);
             }
 
             throw $e;
@@ -282,7 +298,7 @@ class InstallationManager
             pcntl_signal(SIGINT, $prevHandler);
         }
         if ($handleInterruptsWindows) {
-            sapi_windows_set_ctrl_handler($prevHandler, false);
+            sapi_windows_set_ctrl_handler($windowsHandler, false);
         }
 
         // do a last write so that we write the repository even if nothing changed
@@ -292,10 +308,15 @@ class InstallationManager
     }
 
     /**
-     * @param array $operations    List of operations to execute in this batch
-     * @param array $allOperations Complete list of operations to be executed in the install job, used for event listeners
+     * @param OperationInterface[] $operations    List of operations to execute in this batch
+     * @param PromiseInterface[] $cleanupPromises
+     * @param bool $devMode
+     * @param bool $runScripts
+     * @param OperationInterface[] $allOperations Complete list of operations to be executed in the install job, used for event listeners
+     *
+     * @return void
      */
-    private function downloadAndExecuteBatch(RepositoryInterface $repo, array $operations, array &$cleanupPromises, $devMode, $runScripts, array $allOperations)
+    private function downloadAndExecuteBatch(InstalledRepositoryInterface $repo, array $operations, array &$cleanupPromises, $devMode, $runScripts, array $allOperations)
     {
         $promises = array();
 
@@ -308,9 +329,11 @@ class InstallationManager
             }
 
             if ($opType === 'update') {
+                /** @var UpdateOperation $operation */
                 $package = $operation->getTargetPackage();
                 $initialPackage = $operation->getInitialPackage();
             } else {
+                /** @var InstallOperation|MarkAliasInstalledOperation|MarkAliasUninstalledOperation|UninstallOperation $operation */
                 $package = $operation->getPackage();
                 $initialPackage = null;
             }
@@ -344,8 +367,8 @@ class InstallationManager
         $batches = array();
         $batch = array();
         foreach ($operations as $index => $operation) {
-            if (in_array($operation->getOperationType(), array('update', 'install'), true)) {
-                $package = $operation->getOperationType() === 'update' ? $operation->getTargetPackage() : $operation->getPackage();
+            if ($operation instanceof InstallOperation || $operation instanceof UpdateOperation) {
+                $package = $operation instanceof UpdateOperation ? $operation->getTargetPackage() : $operation->getPackage();
                 if ($package->getType() === 'composer-plugin' || $package->getType() === 'composer-installer') {
                     if ($batch) {
                         $batches[] = $batch;
@@ -369,10 +392,15 @@ class InstallationManager
     }
 
     /**
-     * @param array $operations    List of operations to execute in this batch
-     * @param array $allOperations Complete list of operations to be executed in the install job, used for event listeners
+     * @param OperationInterface[] $operations    List of operations to execute in this batch
+     * @param PromiseInterface[] $cleanupPromises
+     * @param bool $devMode
+     * @param bool $runScripts
+     * @param OperationInterface[] $allOperations Complete list of operations to be executed in the install job, used for event listeners
+     *
+     * @return void
      */
-    private function executeBatch(RepositoryInterface $repo, array $operations, array $cleanupPromises, $devMode, $runScripts, array $allOperations)
+    private function executeBatch(InstalledRepositoryInterface $repo, array $operations, array $cleanupPromises, $devMode, $runScripts, array $allOperations)
     {
         $promises = array();
         $postExecCallbacks = array();
@@ -392,9 +420,11 @@ class InstallationManager
             }
 
             if ($opType === 'update') {
+                /** @var UpdateOperation $operation */
                 $package = $operation->getTargetPackage();
                 $initialPackage = $operation->getInitialPackage();
             } else {
+                /** @var InstallOperation|MarkAliasInstalledOperation|MarkAliasUninstalledOperation|UninstallOperation $operation */
                 $package = $operation->getPackage();
                 $initialPackage = null;
             }
@@ -440,30 +470,49 @@ class InstallationManager
             $this->waitOnPromises($promises);
         }
 
+        Platform::workaroundFilesystemIssues();
+
         foreach ($postExecCallbacks as $cb) {
             $cb();
         }
     }
 
+    /**
+     * @param PromiseInterface[] $promises
+     *
+     * @return void
+     */
     private function waitOnPromises(array $promises)
     {
         $progress = null;
-        if ($this->outputProgress && $this->io instanceof ConsoleIO && !$this->io->isDebug() && count($promises) > 1) {
+        if (
+            $this->outputProgress
+            && $this->io instanceof ConsoleIO
+            && !getenv('CI')
+            && !$this->io->isDebug()
+            && count($promises) > 1
+        ) {
             $progress = $this->io->getProgressBar();
         }
         $this->loop->wait($promises, $progress);
         if ($progress) {
             $progress->clear();
+            // ProgressBar in non-decorated output does not output a final line-break and clear() does nothing
+            if (!$this->io->isDecorated()) {
+                $this->io->writeError('');
+            }
         }
     }
 
     /**
      * Executes install operation.
      *
-     * @param RepositoryInterface $repo      repository in which to check
-     * @param InstallOperation    $operation operation instance
+     * @param InstalledRepositoryInterface $repo      repository in which to check
+     * @param InstallOperation             $operation operation instance
+     *
+     * @return PromiseInterface|null
      */
-    public function install(RepositoryInterface $repo, InstallOperation $operation)
+    public function install(InstalledRepositoryInterface $repo, InstallOperation $operation)
     {
         $package = $operation->getPackage();
         $installer = $this->getInstaller($package->getType());
@@ -476,10 +525,12 @@ class InstallationManager
     /**
      * Executes update operation.
      *
-     * @param RepositoryInterface $repo      repository in which to check
-     * @param UpdateOperation     $operation operation instance
+     * @param InstalledRepositoryInterface $repo      repository in which to check
+     * @param UpdateOperation              $operation operation instance
+     *
+     * @return PromiseInterface|null
      */
-    public function update(RepositoryInterface $repo, UpdateOperation $operation)
+    public function update(InstalledRepositoryInterface $repo, UpdateOperation $operation)
     {
         $initial = $operation->getInitialPackage();
         $target = $operation->getTargetPackage();
@@ -492,9 +543,15 @@ class InstallationManager
             $promise = $installer->update($repo, $initial, $target);
             $this->markForNotification($target);
         } else {
-            $this->getInstaller($initialType)->uninstall($repo, $initial);
+            $promise = $this->getInstaller($initialType)->uninstall($repo, $initial);
+            if (!$promise instanceof PromiseInterface) {
+                $promise = \React\Promise\resolve();
+            }
+
             $installer = $this->getInstaller($targetType);
-            $promise = $installer->install($repo, $target);
+            $promise = $promise->then(function () use ($installer, $repo, $target) {
+                return $installer->install($repo, $target);
+            });
         }
 
         return $promise;
@@ -503,10 +560,12 @@ class InstallationManager
     /**
      * Uninstalls package.
      *
-     * @param RepositoryInterface $repo      repository in which to check
-     * @param UninstallOperation  $operation operation instance
+     * @param InstalledRepositoryInterface $repo      repository in which to check
+     * @param UninstallOperation           $operation operation instance
+     *
+     * @return PromiseInterface|null
      */
-    public function uninstall(RepositoryInterface $repo, UninstallOperation $operation)
+    public function uninstall(InstalledRepositoryInterface $repo, UninstallOperation $operation)
     {
         $package = $operation->getPackage();
         $installer = $this->getInstaller($package->getType());
@@ -517,10 +576,12 @@ class InstallationManager
     /**
      * Executes markAliasInstalled operation.
      *
-     * @param RepositoryInterface         $repo      repository in which to check
-     * @param MarkAliasInstalledOperation $operation operation instance
+     * @param InstalledRepositoryInterface $repo      repository in which to check
+     * @param MarkAliasInstalledOperation  $operation operation instance
+     *
+     * @return void
      */
-    public function markAliasInstalled(RepositoryInterface $repo, MarkAliasInstalledOperation $operation)
+    public function markAliasInstalled(InstalledRepositoryInterface $repo, MarkAliasInstalledOperation $operation)
     {
         $package = $operation->getPackage();
 
@@ -532,10 +593,12 @@ class InstallationManager
     /**
      * Executes markAlias operation.
      *
-     * @param RepositoryInterface           $repo      repository in which to check
+     * @param InstalledRepositoryInterface  $repo      repository in which to check
      * @param MarkAliasUninstalledOperation $operation operation instance
+     *
+     * @return void
      */
-    public function markAliasUninstalled(RepositoryInterface $repo, MarkAliasUninstalledOperation $operation)
+    public function markAliasUninstalled(InstalledRepositoryInterface $repo, MarkAliasUninstalledOperation $operation)
     {
         $package = $operation->getPackage();
 
@@ -555,11 +618,19 @@ class InstallationManager
         return $installer->getInstallPath($package);
     }
 
+    /**
+     * @param bool $outputProgress
+     *
+     * @return void
+     */
     public function setOutputProgress($outputProgress)
     {
         $this->outputProgress = $outputProgress;
     }
 
+    /**
+     * @return void
+     */
     public function notifyInstalls(IOInterface $io)
     {
         $promises = array();
@@ -593,10 +664,18 @@ class InstallationManager
 
                 $postData = array('downloads' => array());
                 foreach ($packages as $package) {
-                    $postData['downloads'][] = array(
+                    $packageNotification = array(
                         'name' => $package->getPrettyName(),
                         'version' => $package->getVersion(),
                     );
+                    if (strpos($repoUrl, 'packagist.org/') !== false) {
+                        if (isset(FileDownloader::$downloadMetadata[$package->getName()])) {
+                            $packageNotification['downloaded'] = FileDownloader::$downloadMetadata[$package->getName()];
+                        } else {
+                            $packageNotification['downloaded'] = false;
+                        }
+                    }
+                    $postData['downloads'][] = $packageNotification;
                 }
 
                 $opts = array(
@@ -619,6 +698,9 @@ class InstallationManager
         $this->reset();
     }
 
+    /**
+     * @return void
+     */
     private function markForNotification(PackageInterface $package)
     {
         if ($package->getNotificationUrl()) {
