@@ -14,6 +14,7 @@ namespace Composer\Repository;
 
 use Composer\IO\IOInterface;
 use Composer\Json\JsonFile;
+use Composer\Package\BasePackage;
 use Composer\Package\Loader\ArrayLoader;
 use Composer\Package\Loader\LoaderInterface;
 use Composer\Util\Tar;
@@ -27,10 +28,16 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
     /** @var LoaderInterface */
     protected $loader;
 
+    /** @var string */
     protected $lookup;
+    /** @var array{url: string} */
     protected $repoConfig;
+    /** @var IOInterface */
     private $io;
 
+    /**
+     * @param array{url: string} $repoConfig
+     */
     public function __construct(array $repoConfig, IOInterface $io)
     {
         parent::__construct();
@@ -61,6 +68,11 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
         $this->scanDirectory($this->lookup);
     }
 
+    /**
+     * @param string $path
+     *
+     * @return void
+     */
     private function scanDirectory($path)
     {
         $io = $this->io;
@@ -87,6 +99,9 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
         }
     }
 
+    /**
+     * @return ?BasePackage
+     */
     private function getComposerInformation(\SplFileInfo $file)
     {
         $json = null;
@@ -111,7 +126,7 @@ class ArtifactRepository extends ArrayRepository implements ConfigurableReposito
         }
 
         if (null === $json) {
-            return false;
+            return null;
         }
 
         $package = JsonFile::parseJson($json, $file->getPathname().'#composer.json');
