@@ -21,8 +21,11 @@ use Composer\Downloader\TransportException;
  */
 class AuthHelper
 {
+    /** @var IOInterface */
     protected $io;
+    /** @var Config */
     protected $config;
+    /** @var array<string, string> Map of origins to message displayed */
     private $displayedOriginAuthentications = array();
 
     public function __construct(IOInterface $io, Config $config)
@@ -34,6 +37,8 @@ class AuthHelper
     /**
      * @param string      $origin
      * @param string|bool $storeAuth
+     *
+     * @return void
      */
     public function storeAuth($origin, $storeAuth)
     {
@@ -75,6 +80,7 @@ class AuthHelper
      * @param  string[]    $headers
      * @return array|null  containing retry (bool) and storeAuth (string|bool) keys, if retry is true the request should be
      *                                retried, if storeAuth is true then on a successful retry the authentication should be persisted to auth.json
+     * @phpstan-return ?array{retry: bool, storeAuth: string|bool}
      */
     public function promptAuthIfNeeded($url, $origin, $statusCode, $reason = null, $headers = array())
     {
@@ -116,8 +122,11 @@ class AuthHelper
             $message = "\n".'Could not fetch '.$url.', enter your ' . $origin . ' credentials ' .($statusCode === 401 ? 'to access private repos' : 'to go over the API rate limit');
             $gitLabUtil = new GitLab($this->io, $this->config, null);
 
-            if ($this->io->hasAuthentication($origin) && ($auth = $this->io->getAuthentication($origin)) && in_array($auth['password'], array('gitlab-ci-token', 'private-token', 'oauth2'), true)) {
-                throw new TransportException("Invalid credentials for '" . $url . "', aborting.", $statusCode);
+            if ($this->io->hasAuthentication($origin)) {
+                $auth = $this->io->getAuthentication($origin);
+                if (in_array($auth['password'], array('gitlab-ci-token', 'private-token', 'oauth2'), true)) {
+                    throw new TransportException("Invalid credentials for '" . $url . "', aborting.", $statusCode);
+                }
             }
 
             if (!$gitLabUtil->authorizeOAuth($origin)
@@ -154,7 +163,7 @@ class AuthHelper
         } else {
             // 404s are only handled for github
             if ($statusCode === 404) {
-                return;
+                return null;
             }
 
             // fail if the console is not interactive
@@ -185,10 +194,11 @@ class AuthHelper
     }
 
     /**
-     * @param  array  $headers
-     * @param  string $origin
-     * @param  string $url
-     * @return array  updated headers array
+     * @param string[] $headers
+     * @param string   $origin
+     * @param string   $url
+     *
+     * @return string[] updated headers array
      */
     public function addAuthenticationHeader(array $headers, $origin, $url)
     {
@@ -210,7 +220,7 @@ class AuthHelper
                 if ($auth['password'] === 'oauth2') {
                     $headers[] = 'Authorization: Bearer '.$auth['username'];
                     $authenticationDisplayMessage = 'Using GitLab OAuth token authentication';
-                } elseif ($auth['password'] === 'private-token' || $auth['password'] === 'gitlab-ci-token') {
+                } else {
                     $headers[] = 'PRIVATE-TOKEN: '.$auth['username'];
                     $authenticationDisplayMessage = 'Using GitLab private token authentication';
                 }
