@@ -21,10 +21,13 @@ use Symfony\Component\Process\Process;
  */
 class AllFunctionalTest extends TestCase
 {
+    /** @var string|false */
     protected $oldcwd;
-    protected $oldenv;
-    protected $oldenvCache;
+    /** @var ?string */
     protected $testDir;
+    /**
+     * @var string
+     */
     private static $pharPath;
 
     public function setUp()
@@ -36,7 +39,9 @@ class AllFunctionalTest extends TestCase
 
     public function tearDown()
     {
-        chdir($this->oldcwd);
+        if ($this->oldcwd) {
+            chdir($this->oldcwd);
+        }
 
         if ($this->testDir) {
             $fs = new Filesystem;
@@ -78,7 +83,13 @@ class AllFunctionalTest extends TestCase
             }
         }
 
-        $proc = new Process((defined('PHP_BINARY') ? escapeshellcmd(PHP_BINARY) : 'php').' -dphar.readonly=0 '.escapeshellarg('./bin/compile'), $target);
+        // TODO in v2.3 always call with an array
+        if (method_exists('Symfony\Component\Process\Process', 'fromShellCommandline')) {
+            $proc = new Process(array((defined('PHP_BINARY') ? PHP_BINARY : 'php'), '-dphar.readonly=0', './bin/compile'), $target);
+        } else {
+            // @phpstan-ignore-next-line
+            $proc = new Process((defined('PHP_BINARY') ? escapeshellcmd(PHP_BINARY) : 'php').' -dphar.readonly=0 '.escapeshellarg('./bin/compile'), $target);
+        }
         $exitcode = $proc->run();
 
         if ($exitcode !== 0 || trim($proc->getOutput())) {
@@ -91,6 +102,7 @@ class AllFunctionalTest extends TestCase
     /**
      * @dataProvider getTestFiles
      * @depends testBuildPhar
+     * @param string $testFile
      */
     public function testIntegration($testFile)
     {
@@ -110,8 +122,15 @@ class AllFunctionalTest extends TestCase
             'COMPOSER_CACHE_DIR' => $this->testDir.'cache',
         );
 
-        $cmd = (defined('PHP_BINARY') ? escapeshellcmd(PHP_BINARY) : 'php') .' '.escapeshellarg(self::$pharPath).' --no-ansi '.$testData['RUN'];
-        $proc = new Process($cmd, $this->testDir, $env, null, 300);
+        // TODO in v2.3 always call with an array
+        if (method_exists('Symfony\Component\Process\Process', 'fromShellCommandline')) {
+            $cmd = array((defined('PHP_BINARY') ? PHP_BINARY : 'php'), self::$pharPath, '--no-ansi', $testData['RUN']);
+            $proc = new Process($cmd, $this->testDir, $env, null, 300);
+        } else {
+            $cmd = (defined('PHP_BINARY') ? escapeshellcmd(PHP_BINARY) : 'php') .' '.escapeshellarg(self::$pharPath).' --no-ansi '.$testData['RUN'];
+            // @phpstan-ignore-next-line
+            $proc = new Process($cmd, $this->testDir, $env, null, 300);
+        }
         $output = '';
 
         $exitcode = $proc->run(function ($type, $buffer) use (&$output) {
@@ -123,7 +142,7 @@ class AllFunctionalTest extends TestCase
             $expected = $testData['EXPECT'];
 
             $line = 1;
-            for ($i = 0, $j = 0; $i < strlen($expected); ) {
+            for ($i = 0, $j = 0; $i < strlen($expected);) {
                 if ($expected[$i] === "\n") {
                     $line++;
                 }
@@ -138,7 +157,7 @@ class AllFunctionalTest extends TestCase
                     } else {
                         $this->fail(
                             'Failed to match pattern '.$regex.' at line '.$line.' / abs offset '.$i.': '
-                            .substr($output, $j, min(strpos($output, "\n", $j)-$j, 100)).PHP_EOL.PHP_EOL.
+                            .substr($output, $j, min(strpos($output, "\n", $j) - $j, 100)).PHP_EOL.PHP_EOL.
                             'Output:'.PHP_EOL.$output
                         );
                     }
@@ -146,8 +165,8 @@ class AllFunctionalTest extends TestCase
                 if ($expected[$i] !== $output[$j]) {
                     $this->fail(
                         'Output does not match expectation at line '.$line.' / abs offset '.$i.': '.PHP_EOL
-                        .'-'.substr($expected, $i, min(strpos($expected, "\n", $i)-$i, 100)).PHP_EOL
-                        .'+'.substr($output, $j, min(strpos($output, "\n", $j)-$j, 100)).PHP_EOL.PHP_EOL
+                        .'-'.substr($expected, $i, min(strpos($expected, "\n", $i) - $i, 100)).PHP_EOL
+                        .'+'.substr($output, $j, min(strpos($output, "\n", $j) - $j, 100)).PHP_EOL.PHP_EOL
                         .'Output:'.PHP_EOL.$output
                     );
                 }
@@ -169,6 +188,9 @@ class AllFunctionalTest extends TestCase
         }
     }
 
+    /**
+     * @return array<string, array<string>>
+     */
     public function getTestFiles()
     {
         $tests = array();
@@ -179,9 +201,13 @@ class AllFunctionalTest extends TestCase
         return $tests;
     }
 
+    /**
+     * @param string $file
+     * @return array<string, int|string>
+     */
     private function parseTestFile($file)
     {
-        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file), null, PREG_SPLIT_DELIM_CAPTURE);
+        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file), -1, PREG_SPLIT_DELIM_CAPTURE);
         $data = array();
         $section = null;
 
@@ -237,6 +263,10 @@ class AllFunctionalTest extends TestCase
         return $data;
     }
 
+    /**
+     * @param string $output
+     * @return string
+     */
     private function cleanOutput($output)
     {
         $processed = '';
