@@ -13,10 +13,18 @@
 namespace Composer\Test;
 
 use Composer\Console\Application;
+use Composer\XdebugHandler\XdebugHandler;
 use Symfony\Component\Console\Output\OutputInterface;
 
 class ApplicationTest extends TestCase
 {
+    public function tearDown()
+    {
+        parent::tearDown();
+
+        putenv('COMPOSER_NO_INTERACTION');
+    }
+
     public function testDevWarning()
     {
         $application = new Application;
@@ -54,7 +62,7 @@ class ApplicationTest extends TestCase
         $outputMock->expects($this->at($index++))
             ->method("write");
 
-        if (extension_loaded('xdebug')) {
+        if (XdebugHandler::isXdebugActive()) {
             $outputMock->expects($this->at($index++))
                 ->method("getVerbosity")
                 ->willReturn(OutputInterface::VERBOSITY_NORMAL);
@@ -79,6 +87,10 @@ class ApplicationTest extends TestCase
         $application->doRun($inputMock, $outputMock);
     }
 
+    /**
+     * @param  string $command
+     * @return void
+     */
     public function ensureNoDevWarning($command)
     {
         $application = new Application;
