@@ -29,6 +29,13 @@ class PoolBuilderTest extends TestCase
 {
     /**
      * @dataProvider getIntegrationTests
+     * @param string $file
+     * @param string $message
+     * @param mixed[] $expect
+     * @param mixed[] $root
+     * @param mixed[] $requestData
+     * @param mixed[] $packageRepos
+     * @param mixed[] $fixed
      */
     public function testPoolBuilder($file, $message, $expect, $root, $requestData, $packageRepos, $fixed)
     {
@@ -140,6 +147,9 @@ class PoolBuilderTest extends TestCase
         $this->assertSame($expect, $result);
     }
 
+    /**
+     * @return array<string, array<string>>
+     */
     public function getIntegrationTests()
     {
         $fixturesDir = realpath(__DIR__.'/Fixtures/poolbuilder/');
@@ -173,9 +183,14 @@ class PoolBuilderTest extends TestCase
         return $tests;
     }
 
+    /**
+     * @param \SplFileInfo $file
+     * @param string $fixturesDir
+     * @return array<string, string>
+     */
     protected function readTestFile(\SplFileInfo $file, $fixturesDir)
     {
-        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file->getRealPath()), null, PREG_SPLIT_DELIM_CAPTURE);
+        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file->getRealPath()), -1, PREG_SPLIT_DELIM_CAPTURE);
 
         $sectionInfo = array(
             'TEST' => true,
