@@ -17,11 +17,13 @@ use Composer\EventDispatcher\EventDispatcher;
 use Composer\Installer\InstallerEvents;
 use Composer\Config;
 use Composer\Composer;
+use Composer\IO\IOInterface;
 use Composer\Test\TestCase;
 use Composer\IO\BufferIO;
 use Composer\Script\ScriptEvents;
 use Composer\Script\Event as ScriptEvent;
 use Composer\Util\ProcessExecutor;
+use Composer\Test\Mock\ProcessExecutorMock;
 use Symfony\Component\Console\Output\OutputInterface;
 
 class EventDispatcherTest extends TestCase
@@ -51,12 +53,17 @@ class EventDispatcherTest extends TestCase
     }
 
     /**
-     * @dataProvider getValidCommands
+     * @dataProvider provideValidCommands
+     *
      * @param string $command
      */
     public function testDispatcherCanExecuteSingleCommandLineScript($command)
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
+        $process = new ProcessExecutorMock;
+        $process->expects(array(
+            $command,
+        ), true);
+
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                 $this->createComposerInstance(),
@@ -71,16 +78,14 @@ class EventDispatcherTest extends TestCase
             ->method('getListeners')
             ->will($this->returnValue($listener));
 
-        $process->expects($this->once())
-            ->method('execute')
-            ->with($command)
-            ->will($this->returnValue(0));
-
         $dispatcher->dispatchScript(ScriptEvents::POST_INSTALL_CMD, false);
+
+        $process->assertComplete($this);
     }
 
     /**
-     * @dataProvider getDevModes
+     * @dataProvider provideDevModes
+     *
      * @param bool $devMode
      */
     public function testDispatcherPassDevModeToAutoloadGeneratorForScriptEvents($devMode)
@@ -104,7 +109,7 @@ class EventDispatcherTest extends TestCase
         $dispatcher = new EventDispatcher(
             $composer,
             $this->getMockBuilder('Composer\IO\IOInterface')->getMock(),
-            $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock()
+            new ProcessExecutorMock
         );
 
         $event = $this->getMockBuilder('Composer\Script\Event')
@@ -118,7 +123,7 @@ class EventDispatcherTest extends TestCase
         $dispatcher->hasEventListeners($event);
     }
 
-    public function getDevModes()
+    public function provideDevModes()
     {
         return array(
             array(true),
@@ -126,6 +131,9 @@ class EventDispatcherTest extends TestCase
         );
     }
 
+    /**
+     * @return \PHPUnit\Framework\MockObject\MockObject&\Composer\Autoload\AutoloadGenerator
+     */
     private function getGeneratorMockForDevModePassingTest()
     {
         $generator = $this->getMockBuilder('Composer\Autoload\AutoloadGenerator')
@@ -150,6 +158,9 @@ class EventDispatcherTest extends TestCase
         return $generator;
     }
 
+    /**
+     * @return \PHPUnit\Framework\MockObject\MockObject&\Composer\Repository\RepositoryManager
+     */
     private function getRepositoryManagerMockForDevModePassingTest()
     {
         $rm = $this->getMockBuilder('Composer\Repository\RepositoryManager')
@@ -179,7 +190,7 @@ class EventDispatcherTest extends TestCase
         $dispatcher = new EventDispatcher(
             $composer,
             $io = new BufferIO('', OutputInterface::VERBOSITY_VERBOSE),
-            $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock()
+            new ProcessExecutorMock
         );
 
         $listener = array($this, 'someMethod');
@@ -214,7 +225,12 @@ class EventDispatcherTest extends TestCase
 
     public function testDispatcherCanExecuteCliAndPhpInSameEventScriptStack()
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
+        $process = new ProcessExecutorMock;
+        $process->expects(array(
+            'echo -n foo',
+            'echo -n bar',
+        ), true);
+
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                 $this->createComposerInstance(),
@@ -226,10 +242,6 @@ class EventDispatcherTest extends TestCase
             ))
             ->getMock();
 
-        $process->expects($this->exactly(2))
-            ->method('execute')
-            ->will($this->returnValue(0));
-
         $listeners = array(
             'echo -n foo',
             'Composer\\Test\\EventDispatcher\\EventDispatcherTest::someMethod',
@@ -246,16 +258,17 @@ class EventDispatcherTest extends TestCase
             '> post-install-cmd: Composer\Test\EventDispatcher\EventDispatcherTest::someMethod'.PHP_EOL.
             '> post-install-cmd: echo -n bar'.PHP_EOL;
         $this->assertEquals($expected, $io->getOutput());
+
+        $process->assertComplete($this);
     }
 
     public function testDispatcherCanPutEnv()
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                 $this->createComposerInstance(),
                 $io = new BufferIO('', OutputInterface::VERBOSITY_VERBOSE),
-                $process,
+                new ProcessExecutorMock,
             ))
             ->setMethods(array(
                 'getListeners',
@@ -285,11 +298,10 @@ class EventDispatcherTest extends TestCase
         chdir(__DIR__);
         putenv('COMPOSER_BIN_DIR=' . __DIR__ . sprintf('%svendor%sbin', DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR));
 
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')->setConstructorArgs(array(
                 $this->createComposerInstance(),
                 $io = new BufferIO('', OutputInterface::VERBOSITY_VERBOSE),
-                $process,
+                new ProcessExecutorMock,
             ))->setMethods(array(
                 'getListeners',
             ))->getMock();
@@ -309,6 +321,9 @@ class EventDispatcherTest extends TestCase
         putenv('COMPOSER_BIN_DIR' . ($composerBinDirBkp === false ? '' : '=' . $composerBinDirBkp));
     }
 
+    /**
+     * @return void
+     */
     public static function createsVendorBinFolderChecksEnvDoesNotContainsBin()
     {
         mkdir(__DIR__ . sprintf('%svendor%sbin', DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR), 0700, true);
@@ -321,6 +336,9 @@ class EventDispatcherTest extends TestCase
         self::assertFalse(strpos($val, __DIR__ . sprintf('%svendor%sbin', DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR)));
     }
 
+    /**
+     * @return void
+     */
     public static function createsVendorBinFolderChecksEnvContainsBin()
     {
         $val = getenv('PATH');
@@ -332,6 +350,9 @@ class EventDispatcherTest extends TestCase
         self::assertNotFalse(strpos($val, __DIR__ . sprintf('%svendor%sbin', DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR)));
     }
 
+    /**
+     * @return void
+     */
     public static function getTestEnv()
     {
         $val = getenv('ABC');
@@ -342,7 +363,13 @@ class EventDispatcherTest extends TestCase
 
     public function testDispatcherCanExecuteComposerScriptGroups()
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
+        $process = new ProcessExecutorMock;
+        $process->expects(array(
+            'echo -n foo',
+            'echo -n baz',
+            'echo -n bar',
+        ), true);
+
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                 $composer = $this->createComposerInstance(),
@@ -354,10 +381,6 @@ class EventDispatcherTest extends TestCase
             ))
             ->getMock();
 
-        $process->expects($this->exactly(3))
-            ->method('execute')
-            ->will($this->returnValue(0));
-
         $dispatcher->expects($this->atLeastOnce())
             ->method('getListeners')
             ->will($this->returnCallback(function (Event $event) {
@@ -383,11 +406,17 @@ class EventDispatcherTest extends TestCase
             '> subgroup: echo -n baz'.PHP_EOL.
             '> group: echo -n bar'.PHP_EOL;
         $this->assertEquals($expected, $io->getOutput());
+
+        $process->assertComplete($this);
     }
 
     public function testRecursionInScriptsNames()
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
+        $process = new ProcessExecutorMock;
+        $process->expects(array(
+            'echo Hello '.ProcessExecutor::escape('World'),
+        ), true);
+
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                 $composer = $this->createComposerInstance(),
@@ -399,10 +428,6 @@ class EventDispatcherTest extends TestCase
             ))
             ->getMock();
 
-        $process->expects($this->exactly(1))
-            ->method('execute')
-            ->will($this->returnValue(0));
-
         $dispatcher->expects($this->atLeastOnce())
             ->method('getListeners')
             ->will($this->returnCallback(function (Event $event) {
@@ -419,21 +444,22 @@ class EventDispatcherTest extends TestCase
 
         $dispatcher->dispatch('helloWorld', new ScriptEvent('helloWorld', $composer, $io));
         $expected = "> helloWorld: @hello World".PHP_EOL.
-            "> hello: echo Hello " .escapeshellarg('World').PHP_EOL;
+            "> hello: echo Hello " .$this->getCmd("'World'").PHP_EOL;
 
         $this->assertEquals($expected, $io->getOutput());
+
+        $process->assertComplete($this);
     }
 
     public function testDispatcherDetectInfiniteRecursion()
     {
         $this->setExpectedException('RuntimeException');
 
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
         ->setConstructorArgs(array(
             $composer = $this->createComposerInstance(),
             $io = $this->getMockBuilder('Composer\IO\IOInterface')->getMock(),
-            $process,
+            new ProcessExecutorMock,
         ))
         ->setMethods(array(
             'getListeners',
@@ -457,7 +483,12 @@ class EventDispatcherTest extends TestCase
         $dispatcher->dispatch('root', new ScriptEvent('root', $composer, $io));
     }
 
-    private function getDispatcherStubForListenersTest($listeners, $io)
+    /**
+     * @param array<callable|string> $listeners
+     *
+     * @return \PHPUnit\Framework\MockObject\MockObject&\Composer\EventDispatcher\EventDispatcher
+     */
+    private function getDispatcherStubForListenersTest($listeners, IOInterface $io)
     {
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
@@ -474,7 +505,7 @@ class EventDispatcherTest extends TestCase
         return $dispatcher;
     }
 
-    public function getValidCommands()
+    public function provideValidCommands()
     {
         return array(
             array('phpunit'),
@@ -549,12 +580,11 @@ class EventDispatcherTest extends TestCase
 
     public function testDispatcherInstallerEvents()
     {
-        $process = $this->getMockBuilder('Composer\Util\ProcessExecutor')->getMock();
         $dispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')
             ->setConstructorArgs(array(
                     $this->createComposerInstance(),
                     $this->getMockBuilder('Composer\IO\IOInterface')->getMock(),
-                    $process,
+                    new ProcessExecutorMock,
                 ))
             ->setMethods(array('getListeners'))
             ->getMock();
@@ -568,21 +598,33 @@ class EventDispatcherTest extends TestCase
         $dispatcher->dispatchInstallerEvent(InstallerEvents::PRE_OPERATIONS_EXEC, true, true, $transaction);
     }
 
+    /**
+     * @return void
+     */
     public static function call()
     {
         throw new \RuntimeException();
     }
 
+    /**
+     * @return true
+     */
     public static function someMethod()
     {
         return true;
     }
 
+    /**
+     * @return true
+     */
     public static function someMethod2()
     {
         return true;
     }
 
+    /**
+     * @return Composer
+     */
     private function createComposerInstance()
     {
         $composer = new Composer;
