@@ -58,14 +58,16 @@ class NullIOTest extends TestCase
     {
         $io = new NullIO();
 
-        $this->assertEquals('foo', $io->askConfirmation('bar', 'foo'));
+        $this->assertEquals(false, $io->askConfirmation('bar', false));
     }
 
     public function testAskAndValidate()
     {
         $io = new NullIO();
 
-        $this->assertEquals('foo', $io->askAndValidate('question', 'validator', false, 'foo'));
+        $this->assertEquals('foo', $io->askAndValidate('question', function ($x) {
+            return true;
+        }, null, 'foo'));
     }
 
     public function testSelect()
