@@ -17,6 +17,11 @@ use Composer\Semver\VersionParser;
 
 class InstalledVersionsTest extends TestCase
 {
+    /**
+     * @var string
+     */
+    private $root;
+
     public static function setUpBeforeClass()
     {
         // disable multiple-ClassLoader-based checks of InstalledVersions by making it seem like no
@@ -33,6 +38,9 @@ class InstalledVersionsTest extends TestCase
 
     public function setUp()
     {
+        $this->root = $this->getUniqueTmpDirectory();
+
+        $dir = $this->root;
         InstalledVersions::reload(require __DIR__.'/Repository/Fixtures/installed.php');
     }
 
@@ -47,16 +55,20 @@ class InstalledVersionsTest extends TestCase
             'foo/impl',
             'foo/impl2',
             'foo/replaced',
+            'meta/package',
         );
         $this->assertSame($names, InstalledVersions::getInstalledPackages());
     }
 
     /**
      * @dataProvider isInstalledProvider
+     * @param bool $expected
+     * @param string $name
+     * @param bool $includeDevRequirements
      */
-    public function testIsInstalled($expected, $name, $constraint = null)
+    public function testIsInstalled($expected, $name, $includeDevRequirements = true)
     {
-        $this->assertSame($expected, InstalledVersions::isInstalled($name));
+        $this->assertSame($expected, InstalledVersions::isInstalled($name, $includeDevRequirements));
     }
 
     public static function isInstalledProvider()
@@ -65,15 +77,19 @@ class InstalledVersionsTest extends TestCase
             array(true,  'foo/impl'),
             array(true,  'foo/replaced'),
             array(true,  'c/c'),
+            array(false, 'c/c', false),
             array(true,  '__root__'),
             array(true,  'b/replacer'),
             array(false, 'not/there'),
-            array(false, 'not/there', '^1.0'),
+            array(true,  'meta/package'),
         );
     }
 
     /**
      * @dataProvider satisfiesProvider
+     * @param bool $expected
+     * @param string $name
+     * @param string $constraint
      */
     public function testSatisfies($expected, $name, $constraint)
     {
@@ -112,6 +128,8 @@ class InstalledVersionsTest extends TestCase
 
     /**
      * @dataProvider getVersionRangesProvider
+     * @param string $expected
+     * @param string $name
      */
     public function testGetVersionRanges($expected, $name)
     {
@@ -134,6 +152,8 @@ class InstalledVersionsTest extends TestCase
 
     /**
      * @dataProvider getVersionProvider
+     * @param ?string $expected
+     * @param string $name
      */
     public function testGetVersion($expected, $name)
     {
@@ -156,6 +176,8 @@ class InstalledVersionsTest extends TestCase
 
     /**
      * @dataProvider getPrettyVersionProvider
+     * @param ?string $expected
+     * @param string $name
      */
     public function testGetPrettyVersion($expected, $name)
     {
@@ -187,21 +209,30 @@ class InstalledVersionsTest extends TestCase
         $this->assertSame(array(
             'pretty_version' => 'dev-master',
             'version' => 'dev-master',
+            'type' => 'library',
+            'install_path' => $this->root . '/./',
             'aliases' => array(
                 '1.10.x-dev',
             ),
             'reference' => 'sourceref-by-default',
             'name' => '__root__',
+            'dev' => true,
         ), InstalledVersions::getRootPackage());
     }
 
+    /**
+     * @group legacy
+     */
     public function testGetRawData()
     {
+        $dir = $this->root;
         $this->assertSame(require __DIR__.'/Repository/Fixtures/installed.php', InstalledVersions::getRawData());
     }
 
     /**
      * @dataProvider getReferenceProvider
+     * @param ?string $expected
+     * @param string $name
      */
     public function testGetReference($expected, $name)
     {
@@ -221,4 +252,24 @@ class InstalledVersionsTest extends TestCase
             array(null, 'c/c'),
         );
     }
+
+    public function testGetInstalledPackagesByType()
+    {
+        $names = array(
+            '__root__',
+            'a/provider',
+            'a/provider2',
+            'b/replacer',
+            'c/c',
+        );
+
+        $this->assertSame($names, \Composer\InstalledVersions::getInstalledPackagesByType('library'));
+    }
+
+    public function testGetInstallPath()
+    {
+        $this->assertSame(realpath($this->root), realpath(\Composer\InstalledVersions::getInstallPath('__root__')));
+        $this->assertSame('/foo/bar/vendor/c/c', \Composer\InstalledVersions::getInstallPath('c/c'));
+        $this->assertNull(\Composer\InstalledVersions::getInstallPath('foo/impl'));
+    }
 }
