@@ -20,7 +20,14 @@ use Composer\Test\TestCase;
  */
 class SuggestedPackagesReporterTest extends TestCase
 {
+    /**
+     * @var \PHPUnit\Framework\MockObject\MockObject
+     */
     private $io;
+
+    /**
+     * @var \Composer\Installer\SuggestedPackagesReporter
+     */
     private $suggestedPackagesReporter;
 
     protected function setUp()
@@ -267,6 +274,9 @@ class SuggestedPackagesReporterTest extends TestCase
         $this->suggestedPackagesReporter->output(SuggestedPackagesReporter::MODE_BY_PACKAGE, $repository);
     }
 
+    /**
+     * @return array<string, string>
+     */
     private function getSuggestedPackageArray()
     {
         return array(
@@ -276,10 +286,13 @@ class SuggestedPackagesReporterTest extends TestCase
         );
     }
 
+    /**
+     * @return \Composer\Package\PackageInterface&\PHPUnit\Framework\MockObject\MockObject
+     */
     private function createPackageMock()
     {
         return $this->getMockBuilder('Composer\Package\Package')
-            ->setConstructorArgs(array(md5(mt_rand()), '1.0.0.0', '1.0.0'))
+            ->setConstructorArgs(array(md5((string) mt_rand()), '1.0.0.0', '1.0.0'))
             ->getMock();
     }
 }
