@@ -19,13 +19,17 @@ use Symfony\Component\Console\Command\Command;
 use Symfony\Component\Console\Input\InputArgument;
 use Symfony\Component\Console\Input\InputOption;
 use Composer\IO\BufferIO;
+use Composer\Config;
 use Composer\Json\JsonFile;
 use Composer\Package\Dumper\ArrayDumper;
 use Composer\Util\Filesystem;
 use Composer\Repository\ArrayRepository;
 use Composer\Repository\RepositoryManager;
+use Composer\Repository\RepositoryInterface;
 use Composer\Repository\InstalledArrayRepository;
 use Composer\Package\RootPackageInterface;
+use Composer\Package\BasePackage;
+use Composer\Package\PackageInterface;
 use Composer\Package\Link;
 use Composer\Package\Locker;
 use Composer\Test\Mock\FactoryMock;
@@ -38,7 +42,9 @@ use Symfony\Component\Console\Formatter\OutputFormatter;
 
 class InstallerTest extends TestCase
 {
+    /** @var string */
     protected $prevCwd;
+    /** @var ?string */
     protected $tempComposerHome;
 
     public function setUp()
@@ -50,7 +56,7 @@ class InstallerTest extends TestCase
     public function tearDown()
     {
         chdir($this->prevCwd);
-        if (is_dir($this->tempComposerHome)) {
+        if (isset($this->tempComposerHome) && is_dir($this->tempComposerHome)) {
             $fs = new Filesystem;
             $fs->removeDirectory($this->tempComposerHome);
         }
@@ -58,6 +64,9 @@ class InstallerTest extends TestCase
 
     /**
      * @dataProvider provideInstaller
+     * @param RootPackageInterface&BasePackage $rootPackage
+     * @param RepositoryInterface[] $repositories
+     * @param array[] $options
      */
     public function testInstaller(RootPackageInterface $rootPackage, $repositories, array $options)
     {
@@ -67,6 +76,21 @@ class InstallerTest extends TestCase
             ->setConstructorArgs(array($io))
             ->getMock();
         $config = $this->getMockBuilder('Composer\Config')->getMock();
+        $config->expects($this->any())
+            ->method('get')
+            ->will($this->returnCallback(function ($key) {
+                switch ($key) {
+                    case 'vendor-dir':
+                        return 'foo';
+                    case 'lock':
+                    case 'notify-on-install':
+                        return true;
+                    case 'platform':
+                        return array();
+                }
+
+                throw new \UnexpectedValueException('Unknown key '.$key);
+            }));
 
         $eventDispatcher = $this->getMockBuilder('Composer\EventDispatcher\EventDispatcher')->disableOriginalConstructor()->getMock();
         $httpDownloader = $this->getMockBuilder('Composer\Util\HttpDownloader')->disableOriginalConstructor()->getMock();
@@ -82,6 +106,7 @@ class InstallerTest extends TestCase
         $installationManager = new InstallationManagerMock();
 
         // emulate a writable lock file
+        /** @var ?string $lockData */
         $lockData = null;
         $lockJsonMock = $this->getMockBuilder('Composer\Json\JsonFile')->disableOriginalConstructor()->getMock();
         $lockJsonMock->expects($this->any())
@@ -125,6 +150,10 @@ class InstallerTest extends TestCase
         $this->assertSame($expectedUninstalled, $uninstalled);
     }
 
+    /**
+     * @param  PackageInterface[] $packages
+     * @return mixed[]
+     */
     protected function makePackagesComparable($packages)
     {
         $dumper = new ArrayDumper();
@@ -187,7 +216,19 @@ class InstallerTest extends TestCase
 
     /**
      * @group slow
-     * @dataProvider getSlowIntegrationTests
+     * @dataProvider provideSlowIntegrationTests
+     * @param string $file
+     * @param string $message
+     * @param ?string $condition
+     * @param Config $composerConfig
+     * @param ?mixed[] $lock
+     * @param ?mixed[] $installed
+     * @param string $run
+     * @param mixed[]|false $expectLock
+     * @param ?mixed[] $expectInstalled
+     * @param ?string $expectOutput
+     * @param string $expect
+     * @param int|string $expectResult
      */
     public function testSlowIntegration($file, $message, $condition, $composerConfig, $lock, $installed, $run, $expectLock, $expectInstalled, $expectOutput, $expect, $expectResult)
     {
@@ -195,7 +236,19 @@ class InstallerTest extends TestCase
     }
 
     /**
-     * @dataProvider getIntegrationTests
+     * @dataProvider provideIntegrationTests
+     * @param string $file
+     * @param string $message
+     * @param ?string $condition
+     * @param Config $composerConfig
+     * @param ?mixed[] $lock
+     * @param ?mixed[] $installed
+     * @param string $run
+     * @param mixed[]|false $expectLock
+     * @param ?mixed[] $expectInstalled
+     * @param ?string $expectOutput
+     * @param string $expect
+     * @param int|string $expectResult
      */
     public function testIntegration($file, $message, $condition, $composerConfig, $lock, $installed, $run, $expectLock, $expectInstalled, $expectOutput, $expect, $expectResult)
     {
@@ -378,6 +431,7 @@ class InstallerTest extends TestCase
             $this->assertSame($expectInstalled, $actualInstalled);
         }
 
+        /** @var InstallationManagerMock $installationManager */
         $installationManager = $composer->getInstallationManager();
         $this->assertSame(rtrim($expect), implode("\n", $installationManager->getTrace()));
 
@@ -389,16 +443,20 @@ class InstallerTest extends TestCase
         }
     }
 
-    public function getSlowIntegrationTests()
+    public function provideSlowIntegrationTests()
     {
         return $this->loadIntegrationTests('installer-slow/');
     }
 
-    public function getIntegrationTests()
+    public function provideIntegrationTests()
     {
         return $this->loadIntegrationTests('installer/');
     }
 
+    /**
+     * @param  string $path
+     * @return mixed[]
+     */
     public function loadIntegrationTests($path)
     {
         $fixturesDir = realpath(__DIR__.'/Fixtures/'.$path);
@@ -480,9 +538,13 @@ class InstallerTest extends TestCase
         return $tests;
     }
 
+    /**
+     * @param  string $fixturesDir
+     * @return mixed[]
+     */
     protected function readTestFile(\SplFileInfo $file, $fixturesDir)
     {
-        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file->getRealPath()), null, PREG_SPLIT_DELIM_CAPTURE);
+        $tokens = preg_split('#(?:^|\n*)--([A-Z-]+)--\n#', file_get_contents($file->getRealPath()), -1, PREG_SPLIT_DELIM_CAPTURE);
 
         $sectionInfo = array(
             'TEST' => true,
