@@ -18,7 +18,10 @@ use Composer\Test\TestCase;
 class GitExcludeFilterTest extends TestCase
 {
     /**
-     * @dataProvider patterns
+     * @dataProvider providePatterns
+     *
+     * @param string  $ignore
+     * @param mixed[] $expected
      */
     public function testPatternEscape($ignore, $expected)
     {
@@ -27,7 +30,7 @@ class GitExcludeFilterTest extends TestCase
         $this->assertEquals($expected, $filter->parseGitIgnoreLine($ignore));
     }
 
-    public function patterns()
+    public function providePatterns()
     {
         return array(
             array('app/config/parameters.yml', array('{(?=[^\.])app/(?=[^\.])config/(?=[^\.])parameters\.yml(?=$|/)}', false, false)),
