@@ -20,6 +20,8 @@ class ValidatingArrayLoaderTest extends TestCase
 {
     /**
      * @dataProvider successProvider
+     *
+     * @param array<string, mixed> $config
      */
     public function testLoadSuccess($config)
     {
@@ -194,11 +196,21 @@ class ValidatingArrayLoaderTest extends TestCase
                     'name' => 'npm-asset/angular--core',
                 ),
             ),
+            array( // refs as int or string
+                array(
+                    'name' => 'foo/bar',
+                    'source' => array('url' => 'https://example.org', 'reference' => 1234, 'type' => 'baz'),
+                    'dist' => array('url' => 'https://example.org', 'reference' => 'foobar', 'type' => 'baz'),
+                ),
+            ),
         );
     }
 
     /**
      * @dataProvider errorProvider
+     *
+     * @param array<string, mixed> $config
+     * @param string[]             $expectedErrors
      */
     public function testLoadFailureThrowsException($config, $expectedErrors)
     {
@@ -217,6 +229,9 @@ class ValidatingArrayLoaderTest extends TestCase
 
     /**
      * @dataProvider warningProvider
+     *
+     * @param array<string, mixed> $config
+     * @param string[]             $expectedWarnings
      */
     public function testLoadWarnings($config, $expectedWarnings)
     {
@@ -232,6 +247,10 @@ class ValidatingArrayLoaderTest extends TestCase
 
     /**
      * @dataProvider warningProvider
+     *
+     * @param array<string, mixed> $config
+     * @param string[]             $expectedWarnings
+     * @param bool                 $mustCheck
      */
     public function testLoadSkipsWarningDataWhenIgnoringErrors($config, $expectedWarnings, $mustCheck = true)
     {
@@ -322,6 +341,19 @@ class ValidatingArrayLoaderTest extends TestCase
                     'transport-options : should be an array, string given',
                 ),
             ),
+            array(
+                array(
+                    'name' => 'foo/bar',
+                    'source' => array('url' => '--foo', 'reference' => ' --bar', 'type' => 'baz'),
+                    'dist' => array('url' => ' --foox', 'reference' => '--barx', 'type' => 'baz'),
+                ),
+                array(
+                    'dist.reference : must not start with a "-", "--barx" given',
+                    'dist.url : must not start with a "-", " --foox" given',
+                    'source.reference : must not start with a "-", " --bar" given',
+                    'source.url : must not start with a "-", "--foo" given',
+                ),
+            ),
         ));
     }
 
